/*!
 * @file fi_ioctl.h
 * @brief Definitions for the Fastec Imaging FPGA driver IOCTL interface.
 *
 * Copyright (C) 2010,2015 Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef __FI_IOCTL_H__
#define __FI_IOCTL_H__

//-----------------------------------------------------------------------------
// Include Files
//-----------------------------------------------------------------------------

#include <linux/ioctl.h>    // Needed for the _IOW, etc. stuff used later.
#include "fi_fpga_regs.h"

//-----------------------------------------------------------------------------
// Constant Definitions
//-----------------------------------------------------------------------------

#define FI_IMA_NON_BLOCKING (1 << 0)

//! Enumeration defining the various I2C busses we support.
typedef enum
{
    FI_I2C_MAIN,            //!< Talking to the main board I2C bus.
    FI_I2C_IO,              //!< Talking to the IO board I2C bus.
    FI_I2C_SENSOR           //!< Talking to the sensor board I2C bus.
} FI_I2C_BUS_ID;

typedef enum
{
    FI_I2C_RO,          //!< Read-only I2C access.
    FI_I2C_WO,          //!< Write-only I2C access.
    FI_I2C_RW           //!< Read-write I2C access.
} FI_I2C_ACCESS_TYPE;

//-----------------------------------------------------------------------------
// IOCTL Definitions
//-----------------------------------------------------------------------------

//! This is the Fastec Imaging IOCTL magic number.
#define FI_FPGA_IOC_MAGIC  (0x80 | 'F')

#define FI_FPGA_IOC_RESET       _IO(  FI_FPGA_IOC_MAGIC, 0)
#define FI_FPGA_IOC_SETREG      _IOW( FI_FPGA_IOC_MAGIC, 1, struct FI_REG_ACCESS )
#define FI_FPGA_IOC_QUERYREG        _IOWR(FI_FPGA_IOC_MAGIC, 2, uint32_t )
#define FI_FPGA_IOC_GETREG      _IOWR(FI_FPGA_IOC_MAGIC, 3, struct FI_REG_ACCESS )
#define FI_FPGA_IOC_MODREG      _IOWR(FI_FPGA_IOC_MAGIC, 4, struct FI_REG_MODIFY )
#define FI_FPGA_IOC_READ_IMAGE      _IOWR(FI_FPGA_IOC_MAGIC, 5, struct FI_IMAGE_ACCESS )
#define FI_FPGA_IOC_WRITE_IMAGE     _IOWR(FI_FPGA_IOC_MAGIC, 6, struct FI_IMAGE_ACCESS )
#define FI_FPGA_IOC_GETEVENT        _IOWR(FI_FPGA_IOC_MAGIC, 7, struct FI_EVENT_INFO )
#define FI_FPGA_IOC_GET_IPM_HISTO   _IOWR(FI_FPGA_IOC_MAGIC, 8, int )
#define FI_FPGA_IOC_READ_IM_FIFO    _IOWR(FI_FPGA_IOC_MAGIC, 9, int )
#define FI_FPGA_IOC_I2C_ACCESS      _IOWR(FI_FPGA_IOC_MAGIC, 10, struct FI_I2C_ACCESS )

#define FI_FPGA_IOC_MAXNR   10

//-----------------------------------------------------------------------------
// Type Definitions
//-----------------------------------------------------------------------------

//! Structure for event related information.
typedef struct FI_EVENT_INFO
{
    uint32_t    event_value;    //!< The event source (i.e. interrupt register).
    uint32_t    event_timeout;  //!< Optional period to wait (in milliseconds).
} FI_EVENT_INFO;

//! Structure to access an image in the FPGA's image memory.
typedef struct FI_IMAGE_ACCESS
{
    uint32_t    options;    //!< E.g. FI_IMA_NON_BLOCKING
    uint32_t    control;    //!< IM2PCIE_DMA_CONTROL
    uint32_t    im_address; //!< IM2PCIE_IM_RD_DMA_START_ADDRESS
    uint32_t    im_block_size;  //!< IM2PCIE_IM_RD_DMA_BLOCK_SIZE
    uint32_t    im_burst_size;  //!< IM2PCIE_IM_RD_DMA_BURST_SIZE
    uint32_t    im_bp_thresh;   //!< IM2PCIE_IM_RD_DMA_BP_THRESHOLDS
    uint32_t    pc_address; //!< IM2PCIE_PCIE_WR_DMA_START_ADDRESS
    uint32_t    pc_block_size;  //!< IM2PCIE_PCIE_WR_DMA_BLOCK_SIZE
    uint32_t    pc_burst_size;  //!< IM2PCIE_PCIE_WR_DMA_BURST_SIZE
    uint32_t    pc_burst_per;   //!< IM2PCIE_PCIE_WR_DMA_BURST_PERIOD
    uint32_t    pc_bp_thresh;   //!< IM2PCIE_PCIE_WR_DMA_BP_THRESHOLDS
    uint32_t    frame_dim;  //!< IM2PCIE_DMA_FRAME_DIMENSIONS
} FI_IMAGE_ACCESS;

//! Structure to access a FPGA register.
typedef struct FI_REG_ACCESS
{
    uint32_t    reg_id;     //!< ID of the register we're interested in.
    uint32_t    value;      //!< Value to read/write from/to the register.
} FI_REG_ACCESS;

//! Structure to modify (read, modify, write) a FPGA register.
typedef struct FI_REG_MODIFY
{
    uint32_t    reg_id;     //!< ID of the register we're interested in.
    uint32_t    value;      //!< Value to read/write from/to the register.
    uint32_t    mask;       //!< Mask to apply to the read data value.
} FI_REG_MODIFY;

//! Structure used to do I2C transactions.
typedef struct FI_I2C_ACCESS
{
    uint8_t access;     //!< Access type, one of FI_I2C_ACCESS_TYPE.
    uint8_t bus;        //!< Which bus, one of FI_I2C_BUS_ID.
    uint8_t chip_addr;  //!< Address of the device.
    uint32_t    reg_addr;   //!< Address of the register. For writes, this is sometimes more than just a register address.
    bool    read_only;  //!< Some reads actually require a write first, some don't.
    uint32_t    cmd_count;  //!< Number of bytes in the command data.
    uint32_t    byte_count; //!< Number of bytes in the reg_data
    union
    {
        uint8_t bytes[4];   //!< Up to 4 bytes of data.
        uint32_t word;
    } reg_data;
} FI_I2C_ACCESS;

#endif /* __FI_IOCTL_H__ */

