/*!
 * @file main.c
 * @brief Entry point for the PCI device driver.
 *
 * Copyright (C) 2017 Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------

#include <linux/module.h>       /* Needed by all modules */
#include <linux/kernel.h>   /* We're doing kernel development. */
#include <linux/init.h>     /* Needed for the macros */

#include "fi_pci_dev.h"     /* Our driver specific includes */

//-----------------------------------------------------------------------------
// Constant Definitions
//-----------------------------------------------------------------------------

//! The PCI driver version.
#define FASTEC_PCI_DRIVER_VERSION "0.01"

//! The Device ID of our PCIe device.
#define FASTEC_PCI_DID 0x0000

//! The Vendor ID of our PCIe device.
#define FASTEC_PCI_VID 0x1172

//-----------------------------------------------------------------------------
// Variable Declarations
//-----------------------------------------------------------------------------

//! The image memory bar.
int fi_bar_select = 2;
module_param_named(bar_select, fi_bar_select, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(bar_select, "BAR to use for I/O");

/*!
 * @brief The Device ID table.  
 *
 * The structure holding all the identifiers we are interested in.  
 * These are the PCI vendor IDs for the devices we will support. Currently
 * that means our camera head device.
 */
static const struct pci_device_id fi_pci_ids[] =
{
    { PCI_DEVICE(FASTEC_PCI_VID, FASTEC_PCI_DID) },
    { 0 }
};

/*!
 * @brief The PCI driver structure.
 *
 * This structure contains all the entry points into our driver.
 * We only need to fill out the bits of interest to our driver, 
 * namely:
 *   - name: The driver name
 *   - id_table: Pointer to table of device IDs the driver is interested in.
 *   - probe: This probing function gets called for all PCI devices which match the ID table.
 *   - remove: The remove function gets called whenever a device being handled by this driver is removed.
 */
static struct pci_driver fi_pci_driver_ops =
{
    .name = MODULE_NAME,
    .id_table = fi_pci_ids,
    .probe = fastec_pci_probe,
    .remove = fastec_pci_remove,
};

/*!
 * @brief The initialization/entry function.
 *
 * This is the driver intialization entry point. It should be referenced
 * by the "module_init" declaration.
 */
static int __init fi_pci_mod_init(void)
{
    int rc = 0;

    printk(KERN_INFO FASTEC_PCI_DRIVER_NAME ": Welcome!");

    rc = pci_register_driver(&fi_pci_driver_ops);
    if (rc)
        printk(KERN_ERR FASTEC_PCI_DRIVER_NAME ": PCI driver registration failed");

    return rc;
}

/*!
 * @brief The driver exit routine.
 *
 * This is the function that will be called when the driver is removed.
 * It should be referenced by the "module_exit" declaration.
 */
static void __exit fi_pci_mod_exit(void)
{
    pci_unregister_driver(&fi_pci_driver_ops);
    printk(KERN_DEBUG FASTEC_PCI_DRIVER_NAME ": exiting...");
}

module_init(fi_pci_mod_init);
module_exit(fi_pci_mod_exit);

MODULE_AUTHOR("Fastec Imaging, Inc.");          /* Who wrote this. */
MODULE_DESCRIPTION("Fastec PCI Driver");    /* Helpful info   */
MODULE_DEVICE_TABLE(pci, fi_pci_ids);
MODULE_VERSION(FASTEC_PCI_DRIVER_VERSION);
MODULE_LICENSE("GPL");                  /* Try to get rid of the taint message. */

